/******************************************************************************
 *
 * $Id:$
 *
 * Copyright (C) 2000 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * The custom help code in this file was contributed by Mike Copperwhite.
 */

#include "stdafx.h"
#include "doxbar.h"
#include "Commands.h"
#include "DoxCfg.h"
#include <afxtempl.h>
#include <io.h>
#include <direct.h>
#include <htmlhelp.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCommands

/*! Creates a command object and reads all configuration data from the registry */
CCommands::CCommands()
{
	m_pApplication = NULL;
	m_pApplicationEventsObj = NULL;
	m_pDebuggerEventsObj = NULL;

	m_strCmdTool        = AfxGetApp()->GetProfileString(_T("Configure"), _T("CmdTool"),          _T(""));
	m_strDoxConfigTempl = AfxGetApp()->GetProfileString(_T("Configure"), _T("DoxConfigTempl"),   _T(""));
	m_strHTMLHelpComp   = AfxGetApp()->GetProfileString(_T("Configure"), _T("HTMLHelpCompiler"), _T(""));
	m_strDoxExe         = AfxGetApp()->GetProfileString(_T("Configure"), _T("DoxExe"),           _T(""));
	m_strUsrBatFile     = AfxGetApp()->GetProfileString(_T("Configure"), _T("UsrBatFile"),       _T(""));
	m_curSelection      = AfxGetApp()->GetProfileInt(   _T("Configure"), _T("CurSelection"),     0);
	if (m_curSelection<0 || m_curSelection>=NumCommands)
	{
		m_curSelection=0;
	}

	int i;
    for (i=0;i<NumCommands;i++)
	{
		CString section;
		section.Format(_T("CustomHelp%d"),i+1);
		m_options[i].m_HelpFile      =     AfxGetApp()->GetProfileString(section, _T("HelpFile"),     _T(""));	
	    m_options[i].m_bUseSelection = (BOOL)(AfxGetApp()->GetProfileInt(section, _T("UseSelection"), 1));
	    m_options[i].m_bUseCursor    = (BOOL)(AfxGetApp()->GetProfileInt(section, _T("UseCursor"),    1));
	    m_options[i].m_bUseFileName  = (BOOL)(AfxGetApp()->GetProfileInt(section, _T("UseFileName"),  1));	
	}
}

/*! Destroys the command object */
CCommands::~CCommands()
{
	ASSERT (m_pApplication != NULL);
	m_pApplication->Release();
}

/*! Sets the application object. Code is generated by the addin wizard. */
void CCommands::SetApplicationObject(IApplication* pApplication)
{
	// This function assumes pApplication has already been AddRef'd
	//  for us, which CDSAddIn did in its QueryInterface call
	//  just before it called us.
	m_pApplication = pApplication;

	// Create Application event handlers
	XApplicationEventsObj::CreateInstance(&m_pApplicationEventsObj);
	m_pApplicationEventsObj->AddRef();
	m_pApplicationEventsObj->Connect(m_pApplication);
	m_pApplicationEventsObj->m_pCommands = this;

	// Create Debugger event handler
	CComPtr<IDispatch> pDebugger;
	if (SUCCEEDED(m_pApplication->get_Debugger(&pDebugger)) 
		&& pDebugger != NULL)
	{
		XDebuggerEventsObj::CreateInstance(&m_pDebuggerEventsObj);
		m_pDebuggerEventsObj->AddRef();
		m_pDebuggerEventsObj->Connect(pDebugger);
		m_pDebuggerEventsObj->m_pCommands = this;
	}
}

/*! Do not set events anymore. Code is generated by the addin wizard. */
void CCommands::UnadviseFromEvents()
{
	ASSERT (m_pApplicationEventsObj != NULL);
	m_pApplicationEventsObj->Disconnect(m_pApplication);
	m_pApplicationEventsObj->Release();
	m_pApplicationEventsObj = NULL;

	if (m_pDebuggerEventsObj != NULL)
	{
		// Since we were able to connect to the Debugger events, we
		//  should be able to access the Debugger object again to
		//  unadvise from its events (thus the VERIFY_OK below--see stdafx.h).
		CComPtr<IDispatch> pDebugger;
		VERIFY_OK(m_pApplication->get_Debugger(&pDebugger));
		ASSERT (pDebugger != NULL);
		m_pDebuggerEventsObj->Disconnect(pDebugger);
		m_pDebuggerEventsObj->Release();
		m_pDebuggerEventsObj = NULL;
	}
}


/////////////////////////////////////////////////////////////////////////////
// Event handlers

// TODO: Fill out the implementation for those events you wish handle
//  Use m_pCommands->GetApplicationObject() to access the Developer
//  Studio Application object

// Application events

/*! Event send before the build is start. Does nothing. */
HRESULT CCommands::XApplicationEvents::BeforeBuildStart()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send when a build is finished. Does nothing. */
HRESULT CCommands::XApplicationEvents::BuildFinish(long nNumErrors, long nNumWarnings)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send before the application is shutdown. Does nothing. */
HRESULT CCommands::XApplicationEvents::BeforeApplicationShutDown()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send when a document is opened. Does nothing. */
HRESULT CCommands::XApplicationEvents::DocumentOpen(IDispatch* theDocument)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send before a document is closed. Does nothing. */
HRESULT CCommands::XApplicationEvents::BeforeDocumentClose(IDispatch* theDocument)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send when a document is saved. Does nothing. */
HRESULT CCommands::XApplicationEvents::DocumentSave(IDispatch* theDocument)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send if a new document is created. Does nothing. */ 
HRESULT CCommands::XApplicationEvents::NewDocument(IDispatch* theDocument)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send if a window is activated. Does nothing. */
HRESULT CCommands::XApplicationEvents::WindowActivate(IDispatch* theWindow)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send if a window is deactivated. DOes nothing. */
HRESULT CCommands::XApplicationEvents::WindowDeactivate(IDispatch* theWindow)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send if a workspace is opened. Does nothing. */
HRESULT CCommands::XApplicationEvents::WorkspaceOpen()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send if a workspace is closed. Does nothing. */
HRESULT CCommands::XApplicationEvents::WorkspaceClose()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send if new workspace is created. Does nothing. */
HRESULT CCommands::XApplicationEvents::NewWorkspace()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

/*! Event send if the debugger hits a breakpoint. Does nothing. */
HRESULT CCommands::XDebuggerEvents::BreakpointHit(IDispatch* pBreakpoint)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}


/******************** HELPER FUNCTIONS ******************/

/*! Returns TRUE if name has a known source or header extension. */
static BOOL ExtensionSupported(CString &name)
{
	int i=name.GetLength()-1;
	while (i>=0 && name[i]!='.') i--;
	CString ext = name.Right(name.GetLength()-i-1);
	ext.MakeLower();
	if (ext=="hpp" || ext=="h" || ext=="hh" || ext=="hxx" ||
		ext=="cpp" || ext=="c" || ext=="cc" || ext=="cxx" ||
		ext=="idl"
	   ) 
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/*! Return input string \a name with any path stripped. */
static CString GetBaseName(const char *name)
{
	CString result = name;
	int i=result.ReverseFind('\\');
	return result.Left(i+1);
}

/*! Reads a text file into memory.
 *  \param name Input file name.
 *  \param pResult Result buffer in the form of a string.
 *  \return TRUE if successful, FALSE if the file could not be opened or read.
 */
static BOOL ReadTextFileToString(const CString &name,CString *pResult)
{
	if (pResult==0) return FALSE;
	HANDLE hFile = CreateFile(name,
		                      GENERIC_READ,
						      FILE_SHARE_READ,
							  NULL,
							  OPEN_EXISTING,
							  FILE_ATTRIBUTE_NORMAL,
							  NULL
							 );

	if (hFile==INVALID_HANDLE_VALUE) return FALSE; // file could not be opened!
	
	DWORD  dwSize = GetFileSize (hFile, NULL) ;
	DWORD  dwReadBytes;
	char * lpBuffer = new char[dwSize+1];


    if (lpBuffer==0) return FALSE; // out of memory!

	if (!ReadFile(hFile,        // handle of file to read
                  lpBuffer,     // pointer to buffer that receives data
                  dwSize,       // number of bytes to read
                  &dwReadBytes, // pointer to number of bytes read
                  NULL          // pointer to structure for data
                 )
	   )
	{
		delete lpBuffer;
		CloseHandle(hFile);
	    return FALSE;
	}
	
	lpBuffer[dwSize]='\0'; // add null terminator
	*pResult = CString(lpBuffer);
	delete lpBuffer;
    CloseHandle(hFile);
	return TRUE;
}

/*! Writes a string to a file.
 *  \param name The file name to write to. The file is created.
 *  \param buffer The string to write.
 *  \return TRUE if successful, FALSE if the file could not be opened or read.
 */
static BOOL WriteStringToTextFile(const CString &name,const CString &buffer)
{
	HANDLE hFile = CreateFile(name,
		                      GENERIC_WRITE,
							  0,                // no sharing
							  NULL,
							  CREATE_ALWAYS,
							  FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH,
							  NULL
							 );

	if (hFile==INVALID_HANDLE_VALUE) return FALSE;
    
	DWORD dwBytesWritten;    

	if (!WriteFile(hFile,                 // handle to file to write to
                   (const char *)buffer,  // pointer to data to write to file
                   buffer.GetLength(),	  // number of bytes to write
                   &dwBytesWritten,       // pointer to number of bytes written
		           NULL                   // pointer to structure for overlapped I/O
	              )
       ) return FALSE;
	
	CloseHandle(hFile);
	return TRUE;
}

/*! Extract the value of a tag from a Doxyfile.
 *  \param buffer The config file to search through.
 *  \param tag The tag to look for.
 *  \param def The default value of the tag.
 *  \return The value of the tag. If the tag could not be found or is empty the
 *          default value is returned.
 */
static CString extractTagFromBuffer(CString &buffer,const char *tag,const char *def)
{
	CString result=def;
	int i=buffer.Find(tag);
	if (i!=-1)
	{
		i = buffer.Find('=',i);
		if (i!=-1)
		{
			int e=buffer.Find('\n',i+1);
			result=buffer.Mid(i+1,e-i-1);
			result.TrimRight(" \n\r\t");
			result.TrimLeft(" \t");
			if (result.IsEmpty())
			{
				result=def;
			}
		}
	}
	return result;
}

/***********************************************************************/

/*! Runs doxygen on a project. 
 *  This operation consists of the following steps:
 *  - get the name of the currently selected project.
 *  - read the template config file that the user specified.
 *  - extract the path to the HTML output from the template config file.
 *  - read the projects .dsp file.
 *  - extract the used source files from the .dsp file.
 *  - add an INPUT tag with these files to the template config file.
 *  - write the resulting config file.
 *  - generate a batch file to run doxygen, following by the html help compiler.
 *  - goto the project directory and run the batch file.
 */
STDMETHODIMP CCommands::DoxbarExecute() 
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

    //////////////////////////////////////////////////////////////////////////
    // Get the full name of the active project's dsp file
    //////////////////////////////////////////////////////////////////////////	
    
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

    //First, get a pointer to the dispatch for the Projects collection
	CComPtr<IDispatch> pDispProject;
	HRESULT hr = m_pApplication->get_ActiveProject(&pDispProject);
	if (hr != S_OK) // there is no active project
	{
		::MessageBox(NULL, "No active project! Please select one first.", "Doxbar", MB_OK | MB_ICONINFORMATION);
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

    CComQIPtr<IGenericProject, &IID_IGenericProject> pGenProject(pDispProject);
    CComBSTR bstrFullName;
	pGenProject->get_FullName(&bstrFullName);
	CString cstrFullName(bstrFullName);
    CString cstrBaseName = GetBaseName(cstrFullName);

	m_pApplication->PrintToOutputWindow(bstrFullName);

	//////////////////////////////////////////////////////////////////////////
    // check if doxygen.exe exists at the specified path 
    ////////////////////////////////////////////////////////////////////////// 

	if (m_strDoxConfigTempl.IsEmpty())
	{
		::MessageBox(NULL, "No path to configuration template specified. Please configure first!", 
			               "Doxbar", MB_OK | MB_ICONINFORMATION);
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	//////////////////////////////////////////////////////////////////////////
    // read the template configuration file 
    //////////////////////////////////////////////////////////////////////////    

	// check if the configuration is correct
	if (m_strDoxConfigTempl.IsEmpty())
	{
		::MessageBox(NULL, "No template file specified. Please configure first!", 
			               "Doxbar", MB_OK | MB_ICONINFORMATION);
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	CString cfgBuffer;
	if (!ReadTextFileToString(m_strDoxConfigTempl,&cfgBuffer))
	{
		CString msg=CString("Could not open or read template configuration file ")+
			        m_strDoxConfigTempl+"! Check configuration settings!";
		::MessageBox(NULL, msg, "Doxbar", MB_OK | MB_ICONWARNING);
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;		
	}

	//////////////////////////////////////////////////////////////////////////
    // extract the path the HTML directory from the config file
    //////////////////////////////////////////////////////////////////////////   

	CString outputBaseDir = extractTagFromBuffer(cfgBuffer,"OUTPUT_DIRECTORY",".");
    outputBaseDir.Replace('/','\\');
	CString htmlDir       = extractTagFromBuffer(cfgBuffer,"HTML_OUTPUT","html");
	htmlDir.Replace('/','\\');
    CString outputDir;
	if (htmlDir[1]!=':') // relative HTML dir
	{
		if (outputBaseDir[outputBaseDir.GetLength()-1]!='\\') 
	  		outputDir = outputBaseDir+"\\"+htmlDir;
		else
 			outputDir = outputBaseDir+htmlDir;
	}
	else // absolute HTML dir
	{
		outputDir = htmlDir;
	}

	//////////////////////////////////////////////////////////////////////////
    // read the .dsp file 
    //////////////////////////////////////////////////////////////////////////

	CString dspBuffer;
	if (!ReadTextFileToString(cstrFullName,&dspBuffer))
	{
		CString msg=CString("Could not open or read project file ")+cstrFullName;
		::MessageBox(NULL, msg, "Doxbar", MB_OK | MB_ICONWARNING);
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

    //////////////////////////////////////////////////////////////////////////
    // search for SOURCE= lines and gather their arguments in a list 
    //////////////////////////////////////////////////////////////////////////

	CList<CString,CString&> sourceFiles;
    CString name;

	DWORD dwOffset=0;
	DWORD dwLength = dspBuffer.GetLength();
	while (dwOffset<dwLength)
	{
		name.Empty();
	    char c;
		while (dwOffset<dwLength && dspBuffer.GetAt(dwOffset)!='\n') dwOffset++;
		while (dwOffset<dwLength && ((c=dspBuffer.GetAt(dwOffset))=='\n' || c=='\r')) dwOffset++;
		if (dwOffset<dwLength && dspBuffer.Mid(dwOffset,7)=="SOURCE=") // found tag 
		{
		    dwOffset+=7;
		    while (dwOffset<dwLength && 
				   (c=dspBuffer.GetAt(dwOffset))!='\n' && c!='\r' && c!='\t'
				  ) 
			{
				dwOffset++;
				name+=c; // get argument
			}
			sourceFiles.AddTail(name);
		}
	}

    //////////////////////////////////////////////////////////////////////////
    // add an INPUT line with all sources to the template config file 
    //////////////////////////////////////////////////////////////////////////

	cfgBuffer+="\nGENERATE_HTML     = YES"; // force to generate html
	cfgBuffer+="\nGENERATE_HTMLHELP = YES"; // force to generate html help
	cfgBuffer+="\nINPUT = \\\n";
	// Dump the list elements to the output window.
	POSITION pos = sourceFiles.GetHeadPosition();
    for (int i=0;i < sourceFiles.GetCount();i++)
    {
		CString fileName = sourceFiles.GetNext(pos);
		if (ExtensionSupported(fileName))
		{
			cfgBuffer+=fileName+" \\\n";
			//m_pApplication->PrintToOutputWindow(CComBSTR(fileName));
		}
	}
	
    //////////////////////////////////////////////////////////////////////////
    // write the resulting config file to the output 
    //////////////////////////////////////////////////////////////////////////	
	
	CString cstrOutputConfigFile = cstrBaseName+"Doxyfile";

	if (!WriteStringToTextFile(cstrOutputConfigFile,cfgBuffer))
	{
        CString msg=CString("Could not write configuration file ")+
		   	                cstrOutputConfigFile;
		::MessageBox(NULL, msg, "Doxbar", MB_OK | MB_ICONWARNING);
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	//////////////////////////////////////////////////////////////////////////
	// generate a batch file to run doxygen and the html help compiler in one
	// run
	//////////////////////////////////////////////////////////////////////////

    CString batchStr = "\"" + m_strDoxExe + "\"\r\n"
		               "cd " + outputDir + "\r\n"
					   "\"" + m_strHTMLHelpComp + "\" index.hhp\r\n";
	if (!WriteStringToTextFile(m_strUsrBatFile,batchStr))
	{
        CString msg=CString("Could not write batch file ")+
		   	        m_strUsrBatFile + 
					CString(". Check configuration!");
		::MessageBox(NULL, msg, "Doxbar", MB_OK | MB_ICONWARNING);
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	//////////////////////////////////////////////////////////////////////////
	// change current directory the to project directory
	//////////////////////////////////////////////////////////////////////////

	if (_chdir(cstrBaseName)==-1)
	{
		//CString msg=_strerror("Error changing directory");
		CString msg=_strerror(cstrBaseName);
		::MessageBox(NULL, msg, "Doxbar", MB_OK | MB_ICONWARNING);
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

    //////////////////////////////////////////////////////////////////////////
    // run the batch file on the generated config file
    //////////////////////////////////////////////////////////////////////////	

	hr = m_pApplication->ExecuteCommand(CComBSTR(m_strCmdTool));
	if (hr!=S_OK)
	{
#ifdef _DEBUG
		CComBSTR bstr; 
		GetLastErrorDescription(bstr);
		CString msg=CString("Error while executing user tool: ")+CString(bstr);
#else
		CString msg=CString("Error while executing user tool");
#endif
		::MessageBox(NULL, msg, "Doxbar", MB_OK | MB_ICONINFORMATION);
	}

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
	return S_OK;
}

/*! Configure the doxbar.
 *  Pops up a custom dialog based on the settings stored in the registry.
 *  After competion the new settings are written back to the registry.
 */
STDMETHODIMP CCommands::DoxbarConfig()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	
	m_pApplication->EnableModeless(VARIANT_FALSE);

	DoxCfg configDlg(m_options);
	// stuff all the pre read data values in the dialog before
	// showing it
	configDlg.m_strCmdTool        = m_strCmdTool;
	configDlg.m_strUsrBatFile     = m_strUsrBatFile;
	configDlg.m_strDoxExe         = m_strDoxExe;
	configDlg.m_strHTMLHelpComp   = m_strHTMLHelpComp;
	configDlg.m_strDoxConfigTempl = m_strDoxConfigTempl;
	configDlg.m_curSelection      = m_curSelection;

	configDlg.DoModal();
	
	// get the data values back.
	m_strCmdTool                  = configDlg.m_strCmdTool;          
    m_strUsrBatFile               = configDlg.m_strUsrBatFile;       
    m_strDoxExe                   = configDlg.m_strDoxExe;          
    m_strHTMLHelpComp             = configDlg.m_strHTMLHelpComp;    
    m_strDoxConfigTempl           = configDlg.m_strDoxConfigTempl;
	m_curSelection                = configDlg.m_curSelection;
    
    // Here the data for general configuration of Doxbar is saved for later use...
	AfxGetApp()->WriteProfileString(_T("Configure"), _T("CmdTool"),m_strCmdTool);
	AfxGetApp()->WriteProfileString(_T("Configure"), _T("DoxConfigTempl"),m_strDoxConfigTempl);
	AfxGetApp()->WriteProfileString(_T("Configure"), _T("HTMLHelpCompiler"), m_strHTMLHelpComp);
    AfxGetApp()->WriteProfileString(_T("Configure"), _T("DoxExe"),m_strDoxExe);
    AfxGetApp()->WriteProfileString(_T("Configure"), _T("UsrBatFile"),m_strUsrBatFile);
	AfxGetApp()->WriteProfileInt(   _T("Configure"), _T("CurSelection"),m_curSelection);

	int i;
    for (i=0;i<NumCommands;i++)
	{
		CString section;
		section.Format(_T("CustomHelp%d"),i+1);
		AfxGetApp()->WriteProfileString(section,_T("HelpFile"),m_options[i].m_HelpFile);
		AfxGetApp()->WriteProfileInt(section,_T("UseSelection"),m_options[i].m_bUseSelection);
		AfxGetApp()->WriteProfileInt(section,_T("UseCursor"),m_options[i].m_bUseCursor);
		AfxGetApp()->WriteProfileInt(section,_T("UseFileName"),m_options[i].m_bUseFileName);	
	}

	//m_pApplication->PrintToOutputWindow(CComBSTR(m_strDoxygenExePath));
	//m_pApplication->PrintToOutputWindow(CComBSTR(m_strDoxygenConfigFile));
	
	m_pApplication->EnableModeless(VARIANT_TRUE);
    
	return S_OK;
}

/*! Get the current text selection in DevStudio (may be an empty string).
 */
CString CCommands::GetCurTextSelection()
{
	CComBSTR bStr;
	CComPtr<IDispatch> lpDispActDoc, lpDispSel;
	VERIFY_OK(m_pApplication->get_ActiveDocument(&lpDispActDoc));
	if (lpDispActDoc) 
	{
		CComQIPtr<ITextDocument, &IID_ITextDocument> lpActDoc(lpDispActDoc);
		if (lpActDoc && !FAILED(lpActDoc->get_Selection(&lpDispSel))) 
		{
			CComQIPtr<ITextSelection, &IID_ITextSelection> lpSel(lpDispSel);
			if (lpSel) lpSel->get_Text(&bStr);
		}
	}
	CString ret = bStr;
	ret.TrimLeft();
	ret.TrimRight();
	return ret;
}

/*! Get the word currently under the cursor (may be an empty string).
	Will remove any selection, but should restore cursor to original position.
	May affect the undo/redo stack?
 */
CString CCommands::GetWordUnderCursor()
{
	CComBSTR bStr;
	long current_col=-1, current_line=-1;			// Current column and line in editor
	CComPtr<IDispatch> lpDispActDoc, lpDispSel;
	VERIFY_OK(m_pApplication->get_ActiveDocument(&lpDispActDoc));
	if (lpDispActDoc) 
	{
		CComQIPtr<ITextDocument, &IID_ITextDocument> lpActDoc(lpDispActDoc);

		if (lpActDoc && !FAILED(lpActDoc->get_Selection(&lpDispSel))) 
		{
			CComQIPtr<ITextSelection, &IID_ITextSelection> lpSel(lpDispSel);
			if (lpSel) 
			{
				lpSel->get_CurrentColumn(&current_col);		// Get current document position
				lpSel->get_CurrentLine(&current_line);
				
				lpSel->SelectLine();		// Select whole of current line
		 		lpSel->Untabify();			// Convert tabs to spaces so that column==offset+1 into string
		 		lpSel->get_Text(&bStr);		// Get text of the current selection (untabified current line)
		 		VARIANT_BOOL dummy;
		 		lpActDoc->Undo(&dummy);		// Undo 'Untabify'
				
				if (current_col>0 && current_line>0)		// restore previous cursor position
					lpSel->MoveTo(current_line, current_col, CComVariant(dsMove));	
			}
		}
	}
	CString ret;
	if (current_col>0) 
	{
		CString str = bStr;
		int col = current_col-1;	// index of character under cursor
		if (col < str.GetLength()) 
		{

			int start=-1, end=-1;
			for (int n=col; n>=0; n--) 	// Search for word start
			{
				int cur = str[n];
				if (__iscsym(cur))
					start = n;
				else
					break;
			}
			for (n=col; n<str.GetLength(); n++) 	// Search for word end
			{
				int cur = str[n];
				if (__iscsym(cur))
					end = n;
				else
					break;
			}
			if (start>=0 && end>=0) // Extract word if found
			{
				ret = str.Mid(start, end-start+1);
			}
		}
	}
	return ret;
}

/*! Retrieve name of active document without its path */
CString CCommands::GetActiveFileNameWithoutPath()
{
	CComBSTR bStr;
	CComPtr<IDispatch> lpDispActDoc, lpDispSel;
	VERIFY_OK(m_pApplication->get_ActiveDocument(&lpDispActDoc));
	if (lpDispActDoc) 
	{
		CComQIPtr<ITextDocument, &IID_ITextDocument> lpActDoc(lpDispActDoc);
		if (lpActDoc) lpActDoc->get_Name(&bStr);
	}
	CString ret = bStr;
	ret.TrimLeft();
	ret.TrimRight();
	return ret;
}

/*! Callback from custom help command CustomHelp1 */
STDMETHODIMP CCommands::CustomHelp1()
{
	DoCustomHelp(0);
	return S_OK;
}

/*! Callback from custom help command CustomHelp2 */
STDMETHODIMP CCommands::CustomHelp2()
{
	DoCustomHelp(1);
	return S_OK;
}

/*! Callback from custom help command CustomHelp3 */
STDMETHODIMP CCommands::CustomHelp3()
{
	DoCustomHelp(2);
	return S_OK;
}

/*! Callback from custom help command CustomHelp4 */
STDMETHODIMP CCommands::CustomHelp4()
{
	DoCustomHelp(3);
	return S_OK;
}

/*! Callback from custom help command CustomHelp5 */
STDMETHODIMP CCommands::CustomHelp5()
{
	DoCustomHelp(4);
	return S_OK;
}

/*! Callback from custom help command CustomHelp6 */
STDMETHODIMP CCommands::CustomHelp6()
{
	DoCustomHelp(5);
	return S_OK;
}

/*! Callback from custom help command CustomHelp7 */
STDMETHODIMP CCommands::CustomHelp7()
{
	DoCustomHelp(6);
	return S_OK;
}

/*! Callback from custom help command CustomHelp8 */
STDMETHODIMP CCommands::CustomHelp8()
{
	DoCustomHelp(7);
	return S_OK;
}

/*! Callback from custom help command CustomHelp9 */
STDMETHODIMP CCommands::CustomHelp9()
{
	DoCustomHelp(8);
	return S_OK;
}

/*! Callback from custom help command CustomHelp10 */
STDMETHODIMP CCommands::CustomHelp10()
{
	DoCustomHelp(9);
	return S_OK;
}

/*! Callback from custom help command CustomHelp11 */
STDMETHODIMP CCommands::CustomHelp11()
{
	DoCustomHelp(10);
	return S_OK;
}

/*! Callback from custom help command CustomHelp12 */
STDMETHODIMP CCommands::CustomHelp12()
{
	DoCustomHelp(11);
	return S_OK;
}

/*! Callback from custom help command CustomHelp13 */
STDMETHODIMP CCommands::CustomHelp13()
{
	DoCustomHelp(12);
	return S_OK;
}

/*! Callback from custom help command CustomHelp14 */
STDMETHODIMP CCommands::CustomHelp14()
{
	DoCustomHelp(13);
	return S_OK;
}

/*! Callback from custom help command CustomHelp15 */
STDMETHODIMP CCommands::CustomHelp15()
{
	DoCustomHelp(14);
	return S_OK;
}

/*! Callback from custom help command CustomHelp16 */
STDMETHODIMP CCommands::CustomHelp16()
{
	DoCustomHelp(15);
	return S_OK;
}

/*! Callback from custom help command CustomHelp17 */
STDMETHODIMP CCommands::CustomHelp17()
{
	DoCustomHelp(16);
	return S_OK;
}

/*! Callback from custom help command CustomHelp18 */
STDMETHODIMP CCommands::CustomHelp18()
{
	DoCustomHelp(17);
	return S_OK;
}

/*! Callback from custom help command CustomHelp19 */
STDMETHODIMP CCommands::CustomHelp19()
{
	DoCustomHelp(18);
	return S_OK;
}

/*! Callback from custom help command CustomHelp20 */
STDMETHODIMP CCommands::CustomHelp20()
{
	DoCustomHelp(19);
	return S_OK;
}

/*! Execute a help command
 *  \param index0 A zero based index of the help command to execute
 */
void CCommands::DoCustomHelp(int index0)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));
	CString curword;		// word to lookup in index
	
	if (m_options[index0].m_bUseSelection)
		curword = GetCurTextSelection();
	if (m_options[index0].m_bUseCursor && curword.IsEmpty())
		curword = GetWordUnderCursor();
	if (m_options[index0].m_bUseFileName && curword.IsEmpty())
		curword = GetActiveFileNameWithoutPath();

	CString filename = m_options[index0].m_HelpFile;
	filename.TrimLeft();
	filename.TrimRight();
	CString ext = filename.Right(3);
	bool bWinHelp = ext.CompareNoCase("HLP")==0;	// Check for windows help file extension

//	HWND hwndParent = GetTopWindow(NULL);	// This prevents switching between HTML help and Visual Studio
	HWND hwndParent = NULL;

	if (bWinHelp) 
	{
		if (curword.IsEmpty())
			WinHelp(hwndParent, filename, HELP_FINDER, 0);
		else
			WinHelp(hwndParent, filename, HELP_PARTIALKEY, (DWORD)(LPCTSTR)curword);
	} 
	else 
	{
		HtmlHelp(hwndParent, filename, HH_DISPLAY_TOPIC, NULL);	// Display help window

		if (!curword.IsEmpty()) 
		{		// Only look up in index if we have a string to search for
			HH_AKLINK aklink;
			memset(&aklink, 0, sizeof(aklink));
			aklink.cbStruct = sizeof(aklink);
			aklink.pszKeywords = curword;
			aklink.fIndexOnFail = TRUE;

			HtmlHelp(hwndParent, filename, HH_ALINK_LOOKUP, (DWORD)&aklink);
		}
	}
	
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
}

/*! Handler that is called when the user pressed the search buttom.
 *  This activates the current help command.
 */
STDMETHODIMP CCommands::DoxbarSearch()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
    DoCustomHelp(m_curSelection);
	return S_OK;
}
