/******************************************************************************
 *
 * $Id:$
 *
 * Copyright (C) 1997-2000 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * The custom help code in this file was contributed by Mike Copperwhite.
 */

#if !defined(AFX_COMMANDS_H__7175AFA7_ADB3_11D3_813B_005056820166__INCLUDED_)
#define AFX_COMMANDS_H__7175AFA7_ADB3_11D3_813B_005056820166__INCLUDED_

#include "doxbarTypes.h"

/*! Stores the options for each custom help command */
struct CustomHelpOptions 
{				
	CString			m_HelpFile;			///< Name of help file
	BOOL			m_bUseSelection;	///< If true lookup selection in index
	BOOL			m_bUseCursor;		///< If true lookup word under cursor in index
	BOOL			m_bUseFileName;		///< If true lookup current filename in index
};

/*! Command class, ATL object that provides an implemention for the 
 *  ICommand interface. 
 */
class CCommands : 
	public CComDualImpl<ICommands, &IID_ICommands, &LIBID_Doxbar>, 
	public CComObjectRoot,
	public CComCoClass<CCommands, &CLSID_Commands>
{
	
public:
	enum { NumCommands = 20 };
	CCommands();
	~CCommands();
	void SetApplicationObject(IApplication* m_pApplication);
	IApplication* GetApplicationObject() { return m_pApplication; }
	void UnadviseFromEvents();
	

	BEGIN_COM_MAP(CCommands)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY(ICommands)
	END_COM_MAP()
	DECLARE_NOT_AGGREGATABLE(CCommands)

private:
	CString m_strCmdTool;
	CString m_strUsrBatFile;
	CString m_strDoxExe;
	CString m_strDoxConfigTempl;
	CString m_strHTMLHelpComp;
	int m_curSelection;
	CustomHelpOptions m_options[NumCommands];

protected:
	IApplication* m_pApplication;
	CString GetCurTextSelection();
	CString GetWordUnderCursor();
	CString GetActiveFileNameWithoutPath();



protected:
	// This class template is used as the base class for the Application
	//  event handler object and the Debugger event handler object,
	//  which are declared below.
	template <class IEvents, const IID* piidEvents, const GUID* plibid,
		class XEvents, const CLSID* pClsidEvents>
	class XEventHandler :
		public CComDualImpl<IEvents, piidEvents, plibid>,
		public CComObjectRoot,
		public CComCoClass<XEvents, pClsidEvents>
	{
	public:
		BEGIN_COM_MAP(XEvents)
			COM_INTERFACE_ENTRY(IDispatch)
			COM_INTERFACE_ENTRY_IID(*piidEvents, IEvents)
		END_COM_MAP()
		DECLARE_NOT_AGGREGATABLE(XEvents)
		void Connect(IUnknown* pUnk)
		{ VERIFY(SUCCEEDED(AtlAdvise(pUnk, this, *piidEvents, &m_dwAdvise))); }
		void Disconnect(IUnknown* pUnk)
		{ AtlUnadvise(pUnk, *piidEvents, m_dwAdvise); }

		CCommands* m_pCommands;

	protected:
		DWORD m_dwAdvise;
	};

	// This object handles events fired by the Application object
	class XApplicationEvents : public XEventHandler<IApplicationEvents, 
		&IID_IApplicationEvents, &LIBID_Doxbar, 
		XApplicationEvents, &CLSID_ApplicationEvents>
	{
	public:
		// IApplicationEvents methods
		STDMETHOD(BeforeBuildStart)(THIS);
		STDMETHOD(BuildFinish)(THIS_ long nNumErrors, long nNumWarnings);
		STDMETHOD(BeforeApplicationShutDown)(THIS);
		STDMETHOD(DocumentOpen)(THIS_ IDispatch * theDocument);
		STDMETHOD(BeforeDocumentClose)(THIS_ IDispatch * theDocument);
		STDMETHOD(DocumentSave)(THIS_ IDispatch * theDocument);
		STDMETHOD(NewDocument)(THIS_ IDispatch * theDocument);
		STDMETHOD(WindowActivate)(THIS_ IDispatch * theWindow);
		STDMETHOD(WindowDeactivate)(THIS_ IDispatch * theWindow);
		STDMETHOD(WorkspaceOpen)(THIS);
		STDMETHOD(WorkspaceClose)(THIS);
		STDMETHOD(NewWorkspace)(THIS);
	};
	typedef CComObject<XApplicationEvents> XApplicationEventsObj;
	XApplicationEventsObj* m_pApplicationEventsObj;

	// This object handles events fired by the Application object
	class XDebuggerEvents : public XEventHandler<IDebuggerEvents, 
		&IID_IDebuggerEvents, &LIBID_Doxbar, 
		XDebuggerEvents, &CLSID_DebuggerEvents>
	{
	public:
		// IDebuggerEvents method
	    STDMETHOD(BreakpointHit)(THIS_ IDispatch * pBreakpoint);
	};
	typedef CComObject<XDebuggerEvents> XDebuggerEventsObj;
	XDebuggerEventsObj* m_pDebuggerEventsObj;

public:
	STDMETHOD(DoxbarSearch)();
	// ICommands methods
	STDMETHOD(CustomHelp20)();
	STDMETHOD(CustomHelp19)();
	STDMETHOD(CustomHelp18)();
	STDMETHOD(CustomHelp17)();
	STDMETHOD(CustomHelp16)();
	STDMETHOD(CustomHelp15)();
	STDMETHOD(CustomHelp14)();
	STDMETHOD(CustomHelp13)();
	STDMETHOD(CustomHelp12)();
	STDMETHOD(CustomHelp11)();
	STDMETHOD(CustomHelp10)();
	STDMETHOD(CustomHelp9)();
	STDMETHOD(CustomHelp8)();
	STDMETHOD(CustomHelp7)();
	STDMETHOD(CustomHelp6)();
	STDMETHOD(CustomHelp5)();
	STDMETHOD(CustomHelp4)();
	STDMETHOD(CustomHelp3)();
	STDMETHOD(CustomHelp2)();
	STDMETHOD(CustomHelp1)();
	STDMETHOD(DoxbarConfig)();
	STDMETHOD(DoxbarExecute)();

private:
	static DWORD STDMETHODCALLTYPE FileReaderProc(  LPVOID lpParameter );
	void DumpToOutputWindow(HANDLE h);
	void DoCustomHelp(int index0);
};

typedef CComObject<CCommands> CCommandsObj;

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_COMMANDS_H__7175AFA7_ADB3_11D3_813B_005056820166__INCLUDED)
