/******************************************************************************
 *
 * $Id:$
 *
 * Copyright (C) 2000 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * The custom help code in this file was contributed by Mike Copperwhite.
 */
// AddInMod.cpp : implementation file
//

#include "stdafx.h"
#include "doxbar.h"
#include "DSAddIn.h"
#include "Commands.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*! This is called when the user first loads the add-in, and on start-up
    of each subsequent Developer Studio session.
 */
STDMETHODIMP CDSAddIn::OnConnection(IApplication* pApp, VARIANT_BOOL bFirstTime,
		long dwCookie, VARIANT_BOOL* OnConnection)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	// Store info passed to us
	IApplication* pApplication = NULL;
	if (FAILED(pApp->QueryInterface(IID_IApplication, (void**) &pApplication))
		|| pApplication == NULL)
	{
		*OnConnection = VARIANT_FALSE;
		return S_OK;
	}

	m_dwCookie = dwCookie;

	// Create command dispatch, send info back to DevStudio
	CCommandsObj::CreateInstance(&m_pCommands);
	m_pCommands->AddRef();

	// The QueryInterface above AddRef'd the Application object.  It will
	//  be Release'd in CCommand's destructor.
	m_pCommands->SetApplicationObject(pApplication);

	// (see stdafx.h for the definition of VERIFY_OK)

	VERIFY_OK(pApplication->SetAddInInfo((long) AfxGetInstanceHandle(),
		(LPDISPATCH) m_pCommands, IDR_TOOLBAR_MEDIUM, IDR_TOOLBAR_LARGE, m_dwCookie));
	
	// Inform DevStudio of the commands we implement
	
	//////////////////////////////////////////////////////////////////////
	//  Configure Help Commands
	//////////////////////////////////////////////////////////////////////
	for (int n=0; n<CCommands::NumCommands; n++) 
	{
		CString strMethod;
		strMethod.Format(_T("CustomHelp%d"), n+1);

		CString strCommand = strMethod + _T("\n") + strMethod;
		CString strHelp(_T("Display custom help file"));
		strCommand += _T("\n") + strHelp + _T("\n") + strHelp;

		VARIANT_BOOL bRet;
		CComBSTR bszCmdString(strCommand);
		CComBSTR bszMethod(strMethod);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, -1, m_dwCookie, &bRet));
	}

	//////////////////////////////////////////////////////////////////////
	//  Execute Command/Button
	//////////////////////////////////////////////////////////////////////
	{
			
		// The command name should not be localized to other languages.  The 
		//  tooltip, command description, and other strings related to this
		//  command are stored in the string table (IDS_CMD_STRING) and should
		//  be localized.
		LPCTSTR szCommand = _T("DoxbarExecute");
		VARIANT_BOOL bRet;
		CString strCmdString;
		strCmdString.LoadString(IDS_CMD_STRING_EXE);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("DoxbarExecute"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 0, m_dwCookie, &bRet));
		if (bRet == VARIANT_FALSE)
		{
			// AddCommand failed because a command with this name already
			//  exists.  You may try adding your command under a different name.
			//  Or, you can fail to load as we will do here.
			*OnConnection = VARIANT_FALSE;
			return S_OK;
		}
		
		// Add toolbar buttons only if this is the first time the add-in
		//  is being loaded.  Toolbar buttons are automatically remembered
		//  by Developer Studio from session to session, so we should only
		//  add the toolbar buttons once.
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
		}
	}

	//////////////////////////////////////////////////////////////////////
	//  Configure Command/Button
	//////////////////////////////////////////////////////////////////////
	{
			
		// The command name should not be localized to other languages.  The 
		//  tooltip, command description, and other strings related to this
		//  command are stored in the string table (IDS_CMD_STRING) and should
		//  be localized.
		LPCTSTR szCommand = _T("DoxbarConfig");
		VARIANT_BOOL bRet;
		CString strCmdString;
		strCmdString.LoadString(IDS_CMD_STRING_CFG);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("DoxbarConfig"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 1, m_dwCookie, &bRet));
		if (bRet == VARIANT_FALSE)
		{
			// AddCommand failed because a command with this name already
			//  exists.  You may try adding your command under a different name.
			//  Or, you can fail to load as we will do here.
			*OnConnection = VARIANT_FALSE;
			return S_OK;
		}
		
		// Add toolbar buttons only if this is the first time the add-in
		//  is being loaded.  Toolbar buttons are automatically remembered
		//  by Developer Studio from session to session, so we should only
		//  add the toolbar buttons once.
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
		}
		
	}
	
	//////////////////////////////////////////////////////////////////////
	//  Search Command/Button
	//////////////////////////////////////////////////////////////////////
	{
			
		// The command name should not be localized to other languages.  The 
		//  tooltip, command description, and other strings related to this
		//  command are stored in the string table (IDS_CMD_STRING) and should
		//  be localized.
		LPCTSTR szCommand = _T("DoxbarSearch");
		VARIANT_BOOL bRet;
		CString strCmdString;
		strCmdString.LoadString(IDS_CMD_STRING_SEARCH);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("DoxbarSearch"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 2, m_dwCookie, &bRet));
		if (bRet == VARIANT_FALSE)
		{
			// AddCommand failed because a command with this name already
			//  exists.  You may try adding your command under a different name.
			//  Or, you can fail to load as we will do here.
			*OnConnection = VARIANT_FALSE;
			return S_OK;
		}
		
		// Add toolbar buttons only if this is the first time the add-in
		//  is being loaded.  Toolbar buttons are automatically remembered
		//  by Developer Studio from session to session, so we should only
		//  add the toolbar buttons once.
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
		}
		
	}


	*OnConnection = VARIANT_TRUE;
	return S_OK;
}

/*! This is called on shut-down, and also when the user unloads the add-in. */
STDMETHODIMP CDSAddIn::OnDisconnection(VARIANT_BOOL bLastTime)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	m_pCommands->UnadviseFromEvents();
	m_pCommands->Release();
	m_pCommands = NULL;

	// TODO: Perform any cleanup work here

	return S_OK;
}
