/******************************************************************************
 *
 * $Id:$
 *
 * Copyright (C) 2000 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * The custom help code in this file was contributed by Mike Copperwhite.
 */
// DoxCfg.cpp : implementation file
//

#include "stdafx.h"
#include "doxbar.h"
#include "DoxCfg.h"
#include <commdlg.h>
#include "Commands.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// DoxCfg dialog

/*! Creates a configuration dialog. */
DoxCfg::DoxCfg(CustomHelpOptions *options, CWnd* pParent /*=NULL*/)
	: CDialog(DoxCfg::IDD, pParent)
{
	//{{AFX_DATA_INIT(DoxCfg)
	m_strCmdTool = _T("");
	m_strDoxConfigTempl = _T("");
	m_strDoxExe = _T("");
	m_strHTMLHelpComp = _T("");
	m_strUsrBatFile = _T("");
	m_curHelpCmdIndex = -1;
	//}}AFX_DATA_INIT

	m_options = options;
}


void DoxCfg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DoxCfg)
	DDX_Text(pDX, IDC_CMDTOOLEDIT, m_strCmdTool);
	DDX_Text(pDX, IDC_DOXCFGEDIT, m_strDoxConfigTempl);
	DDX_Text(pDX, IDC_DOXEXEEDIT, m_strDoxExe);
	DDX_Text(pDX, IDC_HELPCOMPEDIT, m_strHTMLHelpComp);
	DDX_Text(pDX, IDC_USRBATEDIT, m_strUsrBatFile);
	DDX_CBIndex(pDX, IDC_HELPCMDCOMBO, m_curHelpCmdIndex);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(DoxCfg, CDialog)
	//{{AFX_MSG_MAP(DoxCfg)
	ON_BN_CLICKED(IDC_DOXCFGBUTTON, OnDoxCfgButton)
	ON_BN_CLICKED(IDC_DOXEXEBUTTON, OnDoxExeButton)
	ON_BN_CLICKED(IDC_HELPCOMPBUTTON, OnHelpCompButton)
	ON_CBN_SELENDOK(IDC_HELPCMDCOMBO, OnSelendokHelpcmdcombo)
	ON_BN_CLICKED(IDC_HELPCMDCHECK1, OnHelpcmdcheck1)
	ON_BN_CLICKED(IDC_HELPCMDCHECK2, OnHelpcmdcheck2)
	ON_BN_CLICKED(IDC_HELPCMDCHECK3, OnHelpcmdcheck3)
	ON_BN_CLICKED(IDC_HELPCMDBUTTON, OnHelpCmdButton)
	ON_EN_CHANGE(IDC_HELPCMDEDIT, OnChangeHelpCmdEdit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// DoxCfg message handlers

/*! Handler that is called when the user presses the OK button. */
void DoxCfg::OnOK() 
{
	// TODO: Add extra validation here
	
	CDialog::OnOK();
}

/*! Handler that is called when the user presses the browse button to
 *  locate a template configuration file. Pops up a standard
 *  file requester.
 */
void DoxCfg::OnDoxCfgButton() 
{
	OPENFILENAME OpenFileName;
	TCHAR        szFile[MAX_PATH] = "\0";

	CEdit *edit = (CEdit *)GetDlgItem(IDC_DOXCFGEDIT);
    
	// Fill in the OPENFILENAME structure to support a template and hook.
	OpenFileName.lStructSize       = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner         = m_hWnd;
    OpenFileName.hInstance         = 0;
    OpenFileName.lpstrFilter       = "Any type (*.*)\0*.*\0\0";
    OpenFileName.lpstrCustomFilter = NULL;
    OpenFileName.nMaxCustFilter    = 0;
    OpenFileName.nFilterIndex      = 0;
    OpenFileName.lpstrFile         = szFile;
    OpenFileName.nMaxFile          = sizeof(szFile);
    OpenFileName.lpstrFileTitle    = NULL;
    OpenFileName.nMaxFileTitle     = 0;
    OpenFileName.lpstrInitialDir   = NULL;
    OpenFileName.lpstrTitle        = "Locate template configuration file";
    OpenFileName.nFileOffset       = 0;
    OpenFileName.nFileExtension    = 0;
    OpenFileName.lpstrDefExt       = NULL;
    OpenFileName.lCustData         = 0; 
	OpenFileName.lpfnHook 		   = 0; 
	OpenFileName.lpTemplateName    = 0;
    OpenFileName.Flags             = OFN_HIDEREADONLY; 

	// Call the common dialog function.
    if (edit && GetOpenFileName(&OpenFileName))
    {
		edit->SetWindowText(OpenFileName.lpstrFile);
	}	
}

/*! Handler that is called when the user presses the browse button to
 *  locate doxygen.exe. Pops up a standard file requester.
 */
void DoxCfg::OnDoxExeButton() 
{
	OPENFILENAME OpenFileName;
	TCHAR        szFile[MAX_PATH] = "\0";

	CEdit *edit = (CEdit *)GetDlgItem(IDC_DOXEXEEDIT);
    
	// Fill in the OPENFILENAME structure to support a template and hook.
	OpenFileName.lStructSize       = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner         = m_hWnd;
    OpenFileName.hInstance         = 0;
    OpenFileName.lpstrFilter       = "Executables (*.exe)\0*.exe\0Any type (*.*)\0*.*\0\0";
    OpenFileName.lpstrCustomFilter = NULL;
    OpenFileName.nMaxCustFilter    = 0;
    OpenFileName.nFilterIndex      = 0;
    OpenFileName.lpstrFile         = szFile;
    OpenFileName.nMaxFile          = sizeof(szFile);
    OpenFileName.lpstrFileTitle    = NULL;
    OpenFileName.nMaxFileTitle     = 0;
    OpenFileName.lpstrInitialDir   = NULL;
    OpenFileName.lpstrTitle        = "Locate doxygen.exe";
    OpenFileName.nFileOffset       = 0;
    OpenFileName.nFileExtension    = 0;
    OpenFileName.lpstrDefExt       = NULL;
    OpenFileName.lCustData         = 0; 
	OpenFileName.lpfnHook 		   = 0; 
	OpenFileName.lpTemplateName    = 0;
    OpenFileName.Flags             = OFN_HIDEREADONLY; 

	// Call the common dialog function.
    if (edit && GetOpenFileName(&OpenFileName))
    {
		edit->SetWindowText(OpenFileName.lpstrFile);
	}
	
}

/*! Handler that is called when the user presses the browse button to
 *  locate the HTML help compiler. Pops up a standard file requester.
 */
void DoxCfg::OnHelpCompButton() 
{
	OPENFILENAME OpenFileName;
	TCHAR        szFile[MAX_PATH] = "\0";

	CEdit *edit = (CEdit *)GetDlgItem(IDC_HELPCOMPEDIT);
    
	// Fill in the OPENFILENAME structure to support a template and hook.
	OpenFileName.lStructSize       = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner         = m_hWnd;
    OpenFileName.hInstance         = 0;
    OpenFileName.lpstrFilter       = "Executables (*.exe)\0*.exe\0Any type (*.*)\0*.*\0\0";
    OpenFileName.lpstrCustomFilter = NULL;
    OpenFileName.nMaxCustFilter    = 0;
    OpenFileName.nFilterIndex      = 0;
    OpenFileName.lpstrFile         = szFile;
    OpenFileName.nMaxFile          = sizeof(szFile);
    OpenFileName.lpstrFileTitle    = NULL;
    OpenFileName.nMaxFileTitle     = 0;
    OpenFileName.lpstrInitialDir   = NULL;
    OpenFileName.lpstrTitle        = "Locate HTML Help compiler (hhc.exe)";
    OpenFileName.nFileOffset       = 0;
    OpenFileName.nFileExtension    = 0;
    OpenFileName.lpstrDefExt       = NULL;
    OpenFileName.lCustData         = 0; 
	OpenFileName.lpfnHook 		   = 0; 
	OpenFileName.lpTemplateName    = 0;
    OpenFileName.Flags             = OFN_HIDEREADONLY; 

	// Call the common dialog function.
    if (edit && GetOpenFileName(&OpenFileName))
    {
		edit->SetWindowText(OpenFileName.lpstrFile);
	}
}

/*! Handler that is called when the dialog opens. This set all the
 *  widget elements to there registered values.
 */

BOOL DoxCfg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	CComboBox *cb = (CComboBox *)GetDlgItem(IDC_HELPCMDCOMBO);
    int i;
	for (i=0;i<CCommands::NumCommands;i++)
	{
	  CString cmd;
	  cmd.Format(_T("CustomHelp%d"),i+1);
	  cb->AddString(cmd);
	}
	cb->SetCurSel(m_curSelection);
	((CEdit   *)GetDlgItem(IDC_HELPCMDEDIT))->SetWindowText(m_options[m_curSelection].m_HelpFile);
	((CButton *)GetDlgItem(IDC_HELPCMDCHECK1))->SetCheck(m_options[m_curSelection].m_bUseSelection);
	((CButton *)GetDlgItem(IDC_HELPCMDCHECK2))->SetCheck(m_options[m_curSelection].m_bUseCursor);
	((CButton *)GetDlgItem(IDC_HELPCMDCHECK3))->SetCheck(m_options[m_curSelection].m_bUseFileName);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/*! Handler that is called when the user selects a new help command. */
void DoxCfg::OnSelendokHelpcmdcombo() 
{
	m_curSelection=((CComboBox *)GetDlgItem(IDC_HELPCMDCOMBO))->GetCurSel();
	((CEdit   *)GetDlgItem(IDC_HELPCMDEDIT))->SetWindowText(m_options[m_curSelection].m_HelpFile);
	((CButton *)GetDlgItem(IDC_HELPCMDCHECK1))->SetCheck(m_options[m_curSelection].m_bUseSelection);
	((CButton *)GetDlgItem(IDC_HELPCMDCHECK2))->SetCheck(m_options[m_curSelection].m_bUseCursor);
	((CButton *)GetDlgItem(IDC_HELPCMDCHECK3))->SetCheck(m_options[m_curSelection].m_bUseFileName);
}

/*! Handler that is called when the user checks the "lookup selection" box. */
void DoxCfg::OnHelpcmdcheck1() 
{
	int i=((CComboBox *)GetDlgItem(IDC_HELPCMDCOMBO))->GetCurSel();
	m_options[i].m_bUseSelection=((CButton *)GetDlgItem(IDC_HELPCMDCHECK1))->GetCheck();
}

/*! Handler that is called when the user checks the "lookup word" box. */
void DoxCfg::OnHelpcmdcheck2() 
{
	int i=((CComboBox *)GetDlgItem(IDC_HELPCMDCOMBO))->GetCurSel();
	m_options[i].m_bUseCursor=((CButton *)GetDlgItem(IDC_HELPCMDCHECK2))->GetCheck();
}

/*! Handler that is called when the user checks the "lookup file" box. */
void DoxCfg::OnHelpcmdcheck3() 
{
	int i=((CComboBox *)GetDlgItem(IDC_HELPCMDCOMBO))->GetCurSel();
	m_options[i].m_bUseFileName=((CButton *)GetDlgItem(IDC_HELPCMDCHECK3))->GetCheck();	
}

/*! Handler that is called when the user locates a HTML Help file. 
 *  Pops up a standard file requester.
 */
void DoxCfg::OnHelpCmdButton() 
{
	OPENFILENAME OpenFileName;
	TCHAR        szFile[MAX_PATH] = "\0";
	int i=((CComboBox *)GetDlgItem(IDC_HELPCMDCOMBO))->GetCurSel();

	CEdit *edit = (CEdit *)GetDlgItem(IDC_HELPCMDEDIT);
    
	// Fill in the OPENFILENAME structure to support a template and hook.
	OpenFileName.lStructSize       = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner         = m_hWnd;
    OpenFileName.hInstance         = 0;
    OpenFileName.lpstrFilter       = "Compressed HTML (*.chm)\0*.chm\0Any type (*.*)\0*.*\0\0";
    OpenFileName.lpstrCustomFilter = NULL;
    OpenFileName.nMaxCustFilter    = 0;
    OpenFileName.nFilterIndex      = 0;
    OpenFileName.lpstrFile         = szFile;
    OpenFileName.nMaxFile          = sizeof(szFile);
    OpenFileName.lpstrFileTitle    = NULL;
    OpenFileName.nMaxFileTitle     = 0;
    OpenFileName.lpstrInitialDir   = NULL;
    OpenFileName.lpstrTitle        = "Locate HTML Help file";
    OpenFileName.nFileOffset       = 0;
    OpenFileName.nFileExtension    = 0;
    OpenFileName.lpstrDefExt       = NULL;
    OpenFileName.lCustData         = 0; 
	OpenFileName.lpfnHook 		   = 0; 
	OpenFileName.lpTemplateName    = 0;
    OpenFileName.Flags             = OFN_HIDEREADONLY; 

	// Call the common dialog function.
    if (edit && GetOpenFileName(&OpenFileName))
    {
		edit->SetWindowText(OpenFileName.lpstrFile);
	}
}

/*! Handler that is called when the user edits the name of the 
 *  help command.
 */
void DoxCfg::OnChangeHelpCmdEdit() 
{
	int i=((CComboBox *)GetDlgItem(IDC_HELPCMDCOMBO))->GetCurSel();	
	((CEdit *)GetDlgItem(IDC_HELPCMDEDIT))->GetWindowText(m_options[i].m_HelpFile);
}
